﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;

namespace Chapter_10_VSPrimer
{
    public class Car :  DependencyObject 
    {

        public delegate void DoorOpenDelegate(bool IsOpen);
        public event DoorOpenDelegate DoorOpen;

        /// <summary>
        /// Initializes a new instance of the <see cref="Car"/> class.
        /// </summary>
        public Car()
        {
            this.DoorOpen += new DoorOpenDelegate(Car_DoorOpen);
        }

        /// <summary>
        /// Handles the door open event.
        /// </summary>
        /// <param name="IsOpen">if set to <c>true</c> [is open].</param>
        void Car_DoorOpen(bool IsOpen)
        {

        }
                
        private int Wheels = 4;
        /// <summary>
        /// Gets or sets the number of wheels.
        /// </summary>
        /// <value>The number of wheels.</value>
        public int NumberOfWheels
        {
            get { return this.Wheels ;}
            set { this.Wheels = value;}
        }

        private int Doors = 4;
        /// <summary>
        /// Gets or sets the number of doors.
        /// </summary>
        /// <value>The number of doors.</value>
        public int NumberOfDoors
        {
            get { return this.Doors ; }
            set { this.Doors = value; }
        }

        /// <summary>
        /// Gets or sets the number of seats.
        /// </summary>
        /// <value>The number of seats.</value>
        public int NumberOfSeats
        {
            get { return (int)GetValue(NumberOfSeatsProperty); }
            set { SetValue(NumberOfSeatsProperty, value); }
        }

        // Using a DependencyProperty as the backing store for NumberOfSeats.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty NumberOfSeatsProperty =
            DependencyProperty.Register("NumberOfSeats", typeof(int), typeof(Car));



        /// <summary>
        /// Gets or sets a value indicating whether this instance is serviced.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is serviced; otherwise, <c>false</c>.
        /// </value>
        public bool IsServiced
        {
            get { return (bool)GetValue(IsServicedProperty); }
            set { SetValue(IsServicedProperty, value); }
        }

        public static readonly DependencyProperty IsServicedProperty =
        DependencyProperty.Register("IsServiced", typeof(bool), typeof(Car),
        new FrameworkPropertyMetadata(false,
        FrameworkPropertyMetadataOptions.None)); 
        

        /// <summary>
        /// A constant type for the model of a car
        /// </summary>
        public enum CarModel
        {
            Mustang,
            Bronco,
            Escort,
            Fairlane,
            Mondeo
        }

        /// <summary>
        /// Gets or sets the model.
        /// </summary>
        /// <value>The model.</value>
        public CarModel Model
        {
            get { return (CarModel)GetValue(ModelProperty); }
            set { SetValue(ModelProperty, value); }
        }

        public static readonly DependencyProperty ModelProperty =
            DependencyProperty.Register("Model", typeof(CarModel), typeof(Car), new PropertyMetadata(CarModel.Fairlane));


        /// <summary>
        /// Opens the door.
        /// </summary>
        public void OpenTheDoor()
        {
            this.DoorOpen(true);
        }







    }
}
