﻿using System;
using System.IO;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Navigation;


using System.Collections.Generic;

namespace Chapter_10_VSPrimer
{
	public partial class Window1
	{
        /// <summary>
        /// Initializes a new instance of the <see cref="Window1"/> class.
        /// </summary>
		public Window1()
		{
			this.InitializeComponent();
			
			// Insert code required on object creation below this point. 
            this.AddHandler(GrandChildControl.TestButtonClickedEvent, new RoutedEventHandler(OnTestButtonClickedEvent)); 
		}

        /// <summary>
        /// Called when [test button clicked event].
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Windows.RoutedEventArgs"/> instance containing the event data.</param>
        void OnTestButtonClickedEvent(object sender, RoutedEventArgs e)
        {
            MessageBox.Show("Event Worked!");
        }



        //Make the MyCar instance global in scope
        private Car MyCar = new Car();


        /// <summary>
        /// A type to declare constant days of the week
        /// </summary>
        public enum DayOfTheWeek
        {
            Sunday,
            Monday,
            Tuesday,
            Wednesday,
            Thursday,
            Friday,
            Saturday
        }

        /// <summary>
        /// Handles the Click event of the btnShowResult control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.RoutedEventArgs"/> instance containing the event data.</param>
        private void btnShowResult_Click(object sender, RoutedEventArgs e)
        {
            string FirstName = "Brennon";
            string LastName = "Williams";
            string CompleteName = FirstName + " " + LastName;

            int MyRealAge = 34;
            int AgeIFeel = 54;
            int TrueAge = (AgeIFeel / 2) + MyRealAge;

            string MyStatement = CompleteName + " " + "feels " + Convert.ToString(TrueAge);
            this.txtResult.Text = MyStatement;

        }

        /// <summary>
        /// Handles the Click event of the btnMakeCar control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.RoutedEventArgs"/> instance containing the event data.</param>
        private void btnMakeCar_Click(object sender, RoutedEventArgs e)
        {
            MyCar.NumberOfDoors = 2;
            MyCar.NumberOfWheels = 4;
            MyCar.NumberOfSeats = 7;

            MyCar.Model = Car.CarModel.Mustang;
            this.btnOpenDoor.IsEnabled = true;

            this.MyCar.DoorOpen += new Car.DoorOpenDelegate(MyCar_DoorOpen);
                       
        }

        /// <summary>
        /// Handlers the door open event.
        /// </summary>
        /// <param name="IsOpen">if set to <c>true</c> [is open].</param>
        void MyCar_DoorOpen(bool IsOpen)
        {
            MessageBox.Show("The Car Door was opened");
        }


        /// <summary>
        /// Handles the Click event of the btnCarDetails control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.RoutedEventArgs"/> instance containing the event data.</param>
        private void btnCarDetails_Click(object sender, RoutedEventArgs e)
        {
            switch (this.MyCar.Model)
            {
                case Car.CarModel.Mustang:
                    MessageBox.Show("The car is a Mustang");
                    break;
                case Car.CarModel.Bronco:
                    MessageBox.Show("The car is a Bronco");
                    break;
                case Car.CarModel.Escort:
                    MessageBox.Show("The car is a Escort");
                    break;
                case Car.CarModel.Fairlane:
                    MessageBox.Show("The car is a Fairlane");
                    break;
                case Car.CarModel.Mondeo:
                    MessageBox.Show("The car is a Mondeo");
                    break;
                default:
                    MessageBox.Show("Error.. the car model is not set");
                    break;
            }
        }

        /// <summary>
        /// Handles the Click event of the btnCallMethods control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.RoutedEventArgs"/> instance containing the event data.</param>
        private void btnCallMethods_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                //Take the number given and make the textbox width
                //change
                int WidthIn = Convert.ToInt32(this.txtResult.Text);
                this.txtResult.Width = ValidateWidth(WidthIn);
            }
            catch (Exception x)
            {
                MessageBox.Show("There is an error : " + x.ToString());
            }

        }

        /// <summary>
        /// Validates the width.
        /// </summary>
        /// <param name="NewWidthValue">The new width value.</param>
        /// <returns></returns>
        private int ValidateWidth(int NewWidthValue)
        {
            //Make sure the number entered is minimum 100, maximum 250
            if (NewWidthValue < 100)
                return 100;
            if (NewWidthValue > 250)
                return 250;
            //Number supplied is valid
            return NewWidthValue;
        }
        
        /// <summary>
        /// Allows you to add two numbers (int) together
        /// </summary>
        /// <param name="NumberOne">The first number to add</param>
        /// <param name="NumberTwo">The second number to add</param>
        /// <returns>The two numbers added together</returns>
        public int AddTwoNumbers(int NumberOne, int NumberTwo)
        {
            int Result = NumberOne + NumberTwo;
            return Result;
        }

        /// <summary>
        /// Adds the two numbers.
        /// </summary>
        /// <param name="NumberOne">The number one.</param>
        /// <param name="NumberTwo">The number two.</param>
        /// <returns></returns>
        public int AddTwoNumbers(string NumberOne, string NumberTwo)
        {
            int Result = Convert.ToInt32(NumberOne) + Convert.ToInt32(NumberTwo);
            return Result;
        }

        /// <summary>
        /// Handles the Click event of the btnOpenDoor control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.RoutedEventArgs"/> instance containing the event data.</param>
        private void btnOpenDoor_Click(object sender, RoutedEventArgs e)
        {
            this.MyCar.OpenTheDoor();        
        }




        //The List definition
        List<int> myIntCollection = new List<int>();

        //The Dictionary definition
        Dictionary<int, string> myDictionary = new Dictionary<int, string>();


        /// <summary>
        /// Handles the Click event of the btnListMaker control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.RoutedEventArgs"/> instance containing the event data.</param>
        private void btnListMaker_Click(object sender, RoutedEventArgs e)
        {
            
            int IntAtPosition2 = 0;
            //Adding only ints to the List<T> collection
            this.myIntCollection.Add(1);
            this.myIntCollection.Add(4);
            this.myIntCollection.Add(Convert.ToInt32("19"));
            this.myIntCollection.Add(IntAtPosition2);
            this.myIntCollection.Add(IntAtPosition2);

            //Checking the collection before adding a duplicate value
            if (!this.myIntCollection.Contains(62))
                this.myIntCollection.Add(62);



            this.myDictionary.Add(4, "The number 4");
            this.myDictionary.Add(14, "The number 14");
            this.myDictionary.Add(24, "The number 24");
            this.myDictionary.Add(34, "The number 34");

            //Adding a new item, checking the Key first
            if (!this.myDictionary.ContainsKey(15))
                this.myDictionary.Add(15, "The number is 15");

            //Adding a new item. check the Value first
            if(!this.myDictionary.ContainsValue("The number 34"))
                this.myDictionary.Add(34, "The number 34");
        }






        //Global Dictionary created for the car yard
        public Dictionary<int, Car> CarsInStock = new Dictionary<int, Car>();

        /// <summary>
        /// Handles the Click event of the btnMakeCars control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.RoutedEventArgs"/> instance containing the event data.</param>
        private void btnMakeCars_Click(object sender, RoutedEventArgs e)
        {
            //Simple list of Client references that will be used as a Key in the collection
            int ClientRef_1 = 5552129;
            int ClientRef_2 = 5553241;
            int ClientRef_3 = 5555481;
            int ClientRef_4 = 5553197;
            int ClientRef_5 = 5558724;

            //Building the collection of clients in the collection
            this.CarsInStock.Add(ClientRef_1, this.MakeNewCar(4, 4, 3, Car.CarModel.Bronco));
            this.CarsInStock.Add(ClientRef_2, this.MakeNewCar(2, 4, 2, Car.CarModel.Mustang));
            this.CarsInStock.Add(ClientRef_3, this.MakeNewCar(2, 4, 4, Car.CarModel.Mustang));
            this.CarsInStock.Add(ClientRef_4, this.MakeNewCar(4, 4, 4, Car.CarModel.Escort));
            this.CarsInStock.Add(ClientRef_5, this.MakeNewCar(4, 4, 3, Car.CarModel.Bronco));
            
            //Allow the cars to be serviced now
            this.btnUpdateServiceStatus.IsEnabled = true;
            
        }

        /// <summary>
        /// Makes the new car.
        /// </summary>
        /// <param name="NumberOfDoors">The number of doors.</param>
        /// <param name="NumberOfWheels">The number of wheels.</param>
        /// <param name="NumberOfSeats">The number of seats.</param>
        /// <param name="CarModel">The car model.</param>
        /// <returns></returns>
        private Car MakeNewCar(int NumberOfDoors, 
            int NumberOfWheels, 
            int NumberOfSeats, 
            Car.CarModel CarModel)
        {
            Car NewCar = new Car();

            NewCar.NumberOfDoors = NumberOfDoors ;
            NewCar.NumberOfWheels = NumberOfWheels  ;
            NewCar.NumberOfSeats = NumberOfSeats ;

            NewCar.Model = CarModel ;

            return NewCar;

        }


        /// <summary>
        /// Handles the Click event of the btnUpdateServiceStatus control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.RoutedEventArgs"/> instance containing the event data.</param>
        private void btnUpdateServiceStatus_Click(object sender, RoutedEventArgs e)
        {
            //Determine which car should be serviced
            Car.CarModel ServiceType;

            if (this.radBronco.IsChecked == true)
                ServiceType = Car.CarModel.Bronco;
            else
                ServiceType = Car.CarModel.Mustang ;


            foreach (Car item in this.CarsInStock.Values)
            {
                if (item.Model == ServiceType)
                    item.IsServiced = true;
            }

            MessageBox.Show("All " + ServiceType.ToString() + "'s are now serviced.");
        }



	}
}