﻿using System;
using System.IO;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Navigation;

namespace Chapter14_TransparentWindow
{
	public partial class Window2
	{
		public Window2()
		{
			this.InitializeComponent();
			
			// Insert code required on object creation below this point.        
		}
               
        //==========================================================================
        // The RoutedEvent which is raised when the CurrentWindowState property
        // changes
        //==========================================================================

        public static readonly RoutedEvent WindowStateChangedEvent = EventManager.RegisterRoutedEvent(
            "WindowStateChanged", RoutingStrategy.Tunnel , typeof(RoutedEventHandler), typeof(Window2));

        // Provide CLR accessors for the event
        public event RoutedEventHandler WindowStateChanged
        {
            add { AddHandler(WindowStateChangedEvent, value); }
            remove { RemoveHandler(WindowStateChangedEvent, value); }
        }


        //==========================================================================
        // The Dependency Property which stores the current state of the Window
        //==========================================================================


        public WINDOW_STATE CurrentWindowState
        {
            get { return (WINDOW_STATE)GetValue(CurrentWindowStateProperty); }
            set { SetValue(CurrentWindowStateProperty, value); }
        }

        public static readonly DependencyProperty CurrentWindowStateProperty =
        DependencyProperty.Register("CurrentWindowState", typeof(WINDOW_STATE), typeof(Window2),
        new FrameworkPropertyMetadata(WINDOW_STATE.Null ,
        FrameworkPropertyMetadataOptions.None,
        new PropertyChangedCallback(CurrentWindowStatePropertyChanged)));

        private static void CurrentWindowStatePropertyChanged(DependencyObject depObj, DependencyPropertyChangedEventArgs e)
        {
            Window2 Window2Obj = depObj as Window2;
            if (Window2Obj != null)
            {                
                RoutedEventArgs newEvent = new RoutedEventArgs(Window2.WindowStateChangedEvent, Window2Obj);
                Window2Obj.RaiseEvent(newEvent);
            }
        }


        //==========================================================================
        // The enum type that describes a Window state
        //==========================================================================


        public enum WINDOW_STATE
        {
            Null =0,
            Closed =1,
            Loaded =2
        }


        //==========================================================================
        // The Window events that are raised, allowing you to set the 
        // CurrentWindowState property
        //==========================================================================


        private void Window_Closed(object sender, EventArgs e)
        {   
            this.CurrentWindowState = WINDOW_STATE.Closed;            
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            this.CurrentWindowState = WINDOW_STATE.Loaded;
        }
	}
}