﻿using System;
using System.IO;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Navigation;

using System.Xml;
using System.Windows.Documents;
using System.Windows.Markup;
using System.Windows.Input;
using Microsoft.Win32;


namespace Chapter_15_XAMLNotepad
{
	public partial class Window1
	{
		public Window1()
		{
			this.InitializeComponent();
			
			// Insert code required on object creation below this point.
            
            this.CommandBindings.Add(new CommandBinding(ApplicationCommands.Save, Save_Executed,Save_CanExecute));
            this.CommandBindings.Add(new CommandBinding(ApplicationCommands.Open, Open_Executed, Open_CanExecute));

        }

        #region"COMMAND HANDLERS"
        void Save_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            // Configure save file dialog box
            SaveFileDialog dlg = new SaveFileDialog();
            dlg.FileName = "Document";
            dlg.DefaultExt = ".xaml";
            dlg.Filter = "XAML Markup file (.xaml)|*.xaml";

            // Show save file dialog box
            Nullable<bool> result = dlg.ShowDialog();

            // Process save file dialog box results
            if (result == true)
            {
                // Save document
                string filename = dlg.FileName;

                //Get all the text in our document
                TextRange DocText = new TextRange(this.rtbXAMLText.Document.ContentStart, this.rtbXAMLText.Document.ContentEnd);

                FileStream sWriter = null;
                try
                {
                    using (sWriter = new FileStream(filename, FileMode.Create))
                    {
                        DocText.Save(sWriter, DataFormats.XamlPackage);
                    }

                    MessageBox.Show("File save complete.",
                        "Save Status",
                        MessageBoxButton.OK,
                        MessageBoxImage.Information,
                        MessageBoxResult.OK);

                }
                catch (Exception ex)
                {
                    MessageBox.Show("File save failed with the following error." +
                        Environment.NewLine +
                        Environment.NewLine +
                        ex.Message.ToString(),
                        "Save Status",
                        MessageBoxButton.OK,
                        MessageBoxImage.Error,
                        MessageBoxResult.OK);
                }
                finally
                {
                    sWriter.Close();
                }
            }

        }

        void Save_CanExecute(object sender, CanExecuteRoutedEventArgs e)
        {
            //Always available
            e.CanExecute = true;
        }

        void Open_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            // Configure open file dialog box
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.FileName = "Document"; 
            dlg.DefaultExt = ".xaml"; 
            dlg.Filter = "XAML Markup file (.xaml)|*.xaml"; 

            // Show open file dialog box
            Nullable<bool> result = dlg.ShowDialog();

            // Process open file dialog box results
            if (result == true)
            {
                // Open document
                string filename = dlg.FileName;

                FileStream  sReader = null;
                try
                {
                    //Clear what is already there
                    this.rtbXAMLText.Document.Blocks.Clear();
                    TextRange DocText = new TextRange(this.rtbXAMLText.Document.ContentStart,
                            this.rtbXAMLText.Document.ContentEnd);

                    if (File.Exists(filename))
                    {
                        using (sReader = new FileStream(filename, FileMode.OpenOrCreate))
                        {
                            DocText.Load(sReader, DataFormats.XamlPackage);
                        }                        
                    }


                }
                catch (Exception ex)
                {
                    MessageBox.Show("File Open failed with the following error." +
                        Environment.NewLine +
                        Environment.NewLine +
                        ex.Message.ToString(),
                        "Open Status",
                        MessageBoxButton.OK,
                        MessageBoxImage.Error,
                        MessageBoxResult.OK);
                }
                finally
                {
                    sReader.Close();
                }
                
            }

        }

        void Open_CanExecute(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = true;
        }

        #endregion



        private void rtbXAMLText_TextChanged(object sender, TextChangedEventArgs e)
        {
            //Make sure the app has completed loading first
            if (!this.IsLoaded )
                return;

            //Get all the text in our document
            TextRange DocText = new TextRange(this.rtbXAMLText.Document.ContentStart, this.rtbXAMLText.Document.ContentEnd);

            //Make sure text is present
            if (string.IsNullOrEmpty(DocText.Text))
                return;

            StringReader stringReader = null;
            XmlTextReader xmlReader = null;
            try
            {
                /*In order to create a UIElement tree from the text 
                 * in our input pad, we need to push it into a XML
                 * format, and then Load that through the XAMLReader object */


                stringReader = new StringReader(DocText.Text);
                xmlReader = new XmlTextReader(stringReader);

                UIElement documentRoot = (UIElement)XamlReader.Load(xmlReader);

                //Add our Element tree to the frame
                this.frmDocRoot.Content = documentRoot;

                //Clean up
                this.txtError.Text = string.Empty;

                
            }
            catch (Exception ex)
            {
                //Simple error display
                if (this.txtError != null)
                    this.txtError.Text = ex.Message.ToString();
                
            }
            finally
            {
                //Close all
                if (stringReader != null) stringReader.Close();

                if (xmlReader != null) xmlReader.Close();
            }

            
        }
	}
}

