﻿using System;
using System.IO;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Navigation;

namespace Chapter_16_MergedDictionaries
{
    public partial class Window1
    {
        public Window1()
        {
            this.InitializeComponent();

            // Insert code required on object creation below this point.
        }

        private void ComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {

            if (sender != null)
            {
                //Get the string of the Selected Item
                string ResourceSelected = ((e.OriginalSource as ComboBox).SelectedItem as ComboBoxItem).Content.ToString();

                //Check to see if the ResourceDictionary already exist in the merged dictionaries
                int CurrentPosition = this.FindResourceDictionaryByName(ResourceSelected);

                //Do we need to load up the Resource?
                if (CurrentPosition == -1)
                {
                    //Load up the Resource as specified by the name in the item selected
                    ResourceDictionary InternalRD = Application.LoadComponent(new Uri(ResourceSelected + ".xaml", UriKind.Relative)) as ResourceDictionary;
                    InternalRD.Source = new Uri(ResourceSelected + ".xaml", UriKind.RelativeOrAbsolute);

                    //Add the Resource to the collection
                    Application.Current.Resources.MergedDictionaries.Add(InternalRD);
                }
                else
                {
                    //Using the MergedDictionaries rules, set the dictionary to the last postion
                    this.RepositionRD(CurrentPosition);
                }

            }
        }


        private int FindResourceDictionaryByName(string RDName)
        {
            int LoopCount = 0;
            string FileNameTest = string.Empty;

            //Search the existing collection of Resources for the Dictionary by name using the Source value
            foreach (ResourceDictionary item in Application.Current.Resources.MergedDictionaries)
            {
                //Get just the Filename and the extension
                FileNameTest = Path.GetFileName(item.Source.OriginalString);

                if (FileNameTest == RDName + ".xaml")
                    break;

                LoopCount++;
            }

            if (LoopCount <= Application.Current.Resources.MergedDictionaries.Count - 1)
            {
                //Send back the current position in the Collection
                return LoopCount;
            }
            else
            {
                //-1 indicates that the Name was not found
                return -1;
            }
        }

        private void RepositionRD(int OldPosition)
        {
            //Check that it isnt already the last ResourceDictionary in the list
            if (OldPosition == Application.Current.Resources.MergedDictionaries.Count)
                return;

            //Get an instance of the Resource Dictionary
            ResourceDictionary RemovedRD = Application.Current.Resources.MergedDictionaries[OldPosition] as ResourceDictionary;

            //Remove it from its previous position
            Application.Current.Resources.MergedDictionaries.RemoveAt(OldPosition);

            //Send it to the last item by adding it back to the collection
            Application.Current.Resources.MergedDictionaries.Add(RemovedRD);
        }

        private void btnLoadSkin_Click(object sender, RoutedEventArgs e)
        {
            //Create a dialog box so we can search for the Skin file
            Microsoft.Win32.OpenFileDialog newDialog = new Microsoft.Win32.OpenFileDialog();
            newDialog.DefaultExt = ".xaml";
            newDialog.Filter = "XAML Files|*.xaml";
            newDialog.InitialDirectory = Environment.CurrentDirectory;

            //Show the dialog
            if (newDialog.ShowDialog() == true)
            {
                //Get a stream for the XAMLReader
                Stream newSkin = newDialog.OpenFile();

                //Try and load the loose xaml file
                ResourceDictionary ExternalRD = System.Windows.Markup.XamlReader.Load(newSkin) as ResourceDictionary;
                if (ExternalRD != null)
                {
                    //Set the Source property
                    ExternalRD.Source = new Uri(newDialog.FileName, UriKind.RelativeOrAbsolute);

                    //Add this RD as the last one in the list
                    Application.Current.Resources.MergedDictionaries.Add(ExternalRD);

                    //Add a new ComboBoxItem so the user can reselect the imported skin
                    ComboBoxItem newItem = new ComboBoxItem();
                    newItem.Content = Path.GetFileNameWithoutExtension(newDialog.FileName);
                    this.cmboSkinSelector.Items.Add(newItem);
                }
            }

            //Close the file stream
            newDialog = null;

        }
    }

}