using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Controls.Primitives;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows;

namespace MyCustomControl
{
    public class AutoComplete_TextBox : System.Windows.Controls.TextBox 
    {
        public AutoComplete_TextBox()
        {
            this.BuildPopUp();
        }

        public AutoComplete_TextBox(List<string> WordDictionary)
        {
            this.STORED_LIST = WordDictionary;
            this.BuildPopUp();
        }

        public List<string> STORED_LIST
        {
            get { return (List<string>)GetValue(STORED_LISTProperty); }
            set { SetValue(STORED_LISTProperty, value); }
        }

        public static readonly DependencyProperty STORED_LISTProperty =
        DependencyProperty.Register("STORED_LIST", typeof(List<string>), typeof(AutoComplete_TextBox),
        new FrameworkPropertyMetadata(null, 
        FrameworkPropertyMetadataOptions.None)); 


        private Popup ListPopUp = new Popup();
        private AutoCompleteListBox ListDisplay = new AutoCompleteListBox();
        private void BuildPopUp()
        {
            ListDisplay.MaxHeight = 80;
            ListDisplay.Width = 150;
            ListDisplay.MouseDown += new System.Windows.Input.MouseButtonEventHandler(ListDisplay_MouseDown);
            ListDisplay.EnterKeyPressed += new AutoCompleteListBox.EnterKeyPressedDelegate(ListDisplay_EnterKeyPressed);

            ListPopUp.MouseDown += new System.Windows.Input.MouseButtonEventHandler(ListPopUp_MouseDown);
            ListPopUp.Child = this.ListDisplay;
            ListPopUp.StaysOpen = false;
            ListPopUp.PopupAnimation = PopupAnimation.Fade;
            ListPopUp.Placement = PlacementMode.Relative;
            ListPopUp.PlacementTarget = this;
            ListPopUp.PlacementRectangle = new Rect(10, 20, 300, 300);
        }

        void ListPopUp_MouseDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if (this.ListDisplay.SelectedItem != null)
            {
                this.Text = this.ListDisplay.SelectedItem.ToString();
                this.ListPopUp.IsOpen = false;
            }
        }

        void ListDisplay_EnterKeyPressed()
        {
            if (this.ListDisplay.SelectedItem != null)
            {
                this.Text = this.ListDisplay.SelectedItem.ToString();
                this.ListPopUp.IsOpen = false;
            }
        }

        void ListDisplay_MouseDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if (this.ListDisplay.SelectedItem != null)
            {
                this.Text = this.ListDisplay.SelectedItem.ToString();
                this.ListPopUp.IsOpen = false;
            }
        }

        /// <summary>
        /// Clear the store of its current values
        /// </summary>
        public void ClearStore()
        {
            this.STORED_LIST.Clear();
        }

        /// <summary>
        /// Supply a new value to add to the store
        /// </summary>
        /// <param name="NewValue">A string representing the new value</param>
        public void AddNewValueToStore(string NewValue)
        {
            //Keep only one copy of the item
            if (!this.STORED_LIST.Contains(NewValue))
                this.STORED_LIST.Add(NewValue);
        }

        /// <summary>
        /// Checks to see if the currently entered text is available from
        /// the Store. If so, display the popup
        /// </summary>
        /// <param name="e"></param>
        protected override void OnKeyUp(KeyEventArgs e)
        {
            base.OnKeyUp(e);

            if (e.Key == Key.Down)
            {
                if (!this.ListPopUp.IsOpen)
                    this.ListPopUp.IsOpen = true;

                this.ListDisplay.SelectedIndex = 0;
                this.ListDisplay.Focus();

            }

            if (this.STORED_LIST.Count > 0)
                this.BuildFilteredList();
        }

        private void BuildFilteredList()
        {
                        
            if (this.Text.Length > 0)
            {

                List<string> FilteredStore = this.STORED_LIST.FindAll(DoMatch);

                if (FilteredStore.Count == 0)
                {
                    this.ListPopUp.IsOpen = false;                    
                    return;
                }
                else
                {
                    this.ListDisplay.Items.Clear();

                    foreach (string var in FilteredStore)
                    {
                        this.ListDisplay.Items.Add(var);
                    }

                    this.ListPopUp.IsOpen = true;
                }
            }
            
        }

        private bool DoMatch(string ValueToTest)
        {
            if (ValueToTest.StartsWith(this.Text, true, null))
                return true;
            else
                return false;
        }


    }


    /// <summary>
    /// Custom listbox class to allow user actions to act as a click
    /// </summary>
    class AutoCompleteListBox : ListBox
    {
        public AutoCompleteListBox()
        {
            this.EnterKeyPressed += delegate() { };
        }

        public delegate void EnterKeyPressedDelegate();
        public event EnterKeyPressedDelegate EnterKeyPressed;

        protected override void OnKeyUp(KeyEventArgs e)
        {
            base.OnKeyUp(e);

            if (e.Key == Key.Enter)
                this.EnterKeyPressed();
        }

        protected override void OnMouseDoubleClick(MouseButtonEventArgs e)
        {
            base.OnMouseDoubleClick(e);
            this.EnterKeyPressed();
        }

        protected override void OnMouseDown(MouseButtonEventArgs e)
        {
            base.OnMouseDown(e);
            this.EnterKeyPressed();
        }
    }

}
