﻿using System;
using System.IO;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Navigation;


namespace Chapter_20_MediaElement
{
	public partial class MediaPlayerController
	{
		public MediaPlayerController()
		{
			this.InitializeComponent();

			// Insert code required on object creation below this point.
            this.VolumeChanged += delegate(double NewVolume) { };
            this.BalanceChanged += delegate(double NewBalance) { };
            this.PlaybackPostionChanged += delegate(double NewPosition) { };
            this.MediaController += delegate(MEDIA_CONTROL Controller) { };
        }

        #region"EVENTS"
        public delegate void VolumeChangedDelegate(double NewVolume);
        public event VolumeChangedDelegate VolumeChanged;

        public delegate void BalanceChangedDelegate(double NewBalance);
        public event BalanceChangedDelegate BalanceChanged;

        public delegate void PlaybackPostionChangedDelegate(double NewPosition);
        public event PlaybackPostionChangedDelegate PlaybackPostionChanged;

        public delegate void MediaControllerDelegate(MEDIA_CONTROL Controller);
        public event MediaControllerDelegate MediaController;
        #endregion

        #region"ENUMS"
        /// <summary>
        /// Represents specific Media Control Elements
        /// </summary>
        public enum MEDIA_CONTROL
        {
            Play = 0,
            Stop =1,
            Pause =2
        }

        /// <summary>
        /// Represents object Enabled states
        /// </summary>
        public enum ENABLED_STATE
        {
            Enabled =0,
            Disabled =1
        }
        #endregion

        #region"PUBLIC METHODS"
        /// <summary>
        /// Set the current state of the Play button.
        /// </summary>
        /// <param name="State">The state value to set</param>
        public void SetPlayButtonState(ENABLED_STATE State)
        {
            switch (State)
            {
                case ENABLED_STATE.Enabled:
                    this.btnPlay.IsEnabled = true;
                    break;
                case ENABLED_STATE.Disabled:
                    this.btnPlay.IsEnabled = false;
                    break;
                default:
                    this.btnPlay.IsEnabled = false;
                    break;
            }
        }

        /// <summary>
        /// Set the current state of the Stop button.
        /// </summary>
        /// <param name="State">The state value to set</param>
        public void SetStopButtonState(ENABLED_STATE State)
        {
            switch (State)
            {
                case ENABLED_STATE.Enabled:
                    this.btnStop.IsEnabled = true;
                    break;
                case ENABLED_STATE.Disabled:
                    this.btnStop.IsEnabled = false;
                    break;
                default:
                    this.btnStop.IsEnabled = false;
                    break;
            }
        }

        /// <summary>
        /// Set the current state of the Pause button.
        /// </summary>
        /// <param name="State">The state value to set</param>
        public void SetPauseButtonState(ENABLED_STATE State)
        {
            switch (State)
            {
                case ENABLED_STATE.Enabled:
                    this.btnPause.IsEnabled = true;
                    break;
                case ENABLED_STATE.Disabled:
                    this.btnPause.IsEnabled = false;
                    break;
                default:
                    this.btnPause.IsEnabled = false;
                    break;
            }
        }

        /// <summary>
        /// Set the current state of all Slider elements.
        /// </summary>
        /// <param name="State">The state value to set</param>
        public void SetPlayerControlStates(ENABLED_STATE State)
        {
            switch (State)
            {
                case ENABLED_STATE.Enabled:
                    this.slidBalance.IsEnabled = true;
                    this.slidVolume.IsEnabled = true;
                    this.slidPosition.IsEnabled = true;
                    break;
                case ENABLED_STATE.Disabled:
                    this.slidBalance.IsEnabled = false;
                    this.slidVolume.IsEnabled = false;
                    this.slidPosition.IsEnabled = false;
                    break;
                default:
                    this.slidBalance.IsEnabled = false;
                    this.slidVolume.IsEnabled = false;
                    this.slidPosition.IsEnabled = false;
                    break;
            }
        }

                
        
        private bool UserUpdatePosition = false;

        /// <summary>
        /// Instruct the player to update the playback position slider element
        /// </summary>
        /// <param name="CurrentPostion">The position on the slider that represents
        /// the current playback position</param>
        /// <param name="TotalTime">The total duration of the current media</param>
        public void SetPlayerPostion(Double CurrentPostion, Double TotalTime)
        {
            if (UserUpdatePosition)
                return;
                        
            this.slidPosition.Maximum = TotalTime;
            this.slidPosition.Value = CurrentPostion;
        }

     
        #endregion

        #region"PRIVATE METHODS & EVENT HANDLERS"

        private void slidVolume_ValueChanged(object sender, RoutedEventArgs e)
        {
            if (this.VolumeChanged != null)
                this.VolumeChanged(this.slidVolume.Value);
        }

        private void slidBalance_ValueChanged(object sender, RoutedEventArgs e)
        {
            this.BalanceChanged(this.slidBalance.Value);
        }

        private void btnPause_Click(object sender, RoutedEventArgs e)
        {            
            this.MediaController(MEDIA_CONTROL.Pause);
        }

        private void btnStop_Click(object sender, RoutedEventArgs e)
        {
            this.MediaController(MEDIA_CONTROL.Stop );
        }

        private void btnPlay_Click(object sender, RoutedEventArgs e)
        {
            this.MediaController(MEDIA_CONTROL.Play );
        }
   
        private void slidPosition_PreviewMouseDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {            
            UserUpdatePosition = true;
        }

        private void slidPosition_PreviewMouseUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            this.PlaybackPostionChanged(this.slidPosition.Value);
            UserUpdatePosition = false;
        }
        #endregion

    }
}