﻿using System;
using System.IO;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Navigation;


using Microsoft.Win32;
using System.Windows.Threading;


namespace Chapter_20_MediaElement
{
	public partial class Window1
	{
		public Window1()
		{
			this.InitializeComponent();
			
			// Insert code required on object creation below this point.
            this.MediaPlayerController.VolumeChanged += new MediaPlayerController.VolumeChangedDelegate(MediaPlayerController_VolumeChanged);
            this.MediaPlayerController.BalanceChanged += new MediaPlayerController.BalanceChangedDelegate(MediaPlayerController_BalanceChanged);
            this.MediaPlayerController.PlaybackPostionChanged += new MediaPlayerController.PlaybackPostionChangedDelegate(MediaPlayerController_PlaybackPostionChanged);
            this.MediaPlayerController.MediaController += new MediaPlayerController.MediaControllerDelegate(MediaPlayerController_MediaController);

            CompositionTarget.Rendering += new EventHandler(CompositionTarget_Rendering);
		}

        
        void MediaPlayerController_MediaController(MediaPlayerController.MEDIA_CONTROL Controller)
        {
            // Based on the event parameter, perform an action on the player and
            //control the buttons that the user can click on etc.
            switch (Controller)
            {
                case MediaPlayerController.MEDIA_CONTROL.Play:
                    this.TryPlayMedia();
                    break;
                case MediaPlayerController.MEDIA_CONTROL.Stop:
                    this.mediaMainScreen.Stop();
                                        
                    this.MediaPlayerController.SetPlayButtonState(MediaPlayerController.ENABLED_STATE.Enabled );
                    this.MediaPlayerController.SetStopButtonState(MediaPlayerController.ENABLED_STATE.Disabled );

                    if (this.mediaMainScreen.CanPause)
                        this.MediaPlayerController.SetPauseButtonState(MediaPlayerController.ENABLED_STATE.Disabled );

                    break;
                case MediaPlayerController.MEDIA_CONTROL.Pause:
                    if (this.mediaMainScreen.CanPause)
                    {
                        this.mediaMainScreen.Pause();

                        this.MediaPlayerController.SetPlayButtonState(MediaPlayerController.ENABLED_STATE.Enabled);
                        this.MediaPlayerController.SetStopButtonState(MediaPlayerController.ENABLED_STATE.Enabled );
                        this.MediaPlayerController.SetPauseButtonState(MediaPlayerController.ENABLED_STATE.Disabled);
                    }

                    break;
                default:
                    break;
            }
        }

        void MediaPlayerController_PlaybackPostionChanged(double NewPosition)
        {
            this.mediaMainScreen.Position = new TimeSpan(0, 0, 0, 0, Convert.ToInt32(NewPosition));
        }

        void MediaPlayerController_BalanceChanged(double NewBalance)
        {
            this.mediaMainScreen.Balance = NewBalance;
        }

        void MediaPlayerController_VolumeChanged(double NewVolume)
        {
            this.mediaMainScreen.Volume = NewVolume;
        }

        private void MenuItem_Click(object sender, RoutedEventArgs e)
        {
            //Ask the user to specify a media file to play
            OpenFileDialog OFD = new OpenFileDialog();
            OFD.AddExtension = true;
            OFD.Multiselect = false;
            OFD.Title = "Select Media File to Play";
            OFD.DefaultExt = "wmv";

            //Add as many filters as you like using the following format
            OFD.Filter = "Windows Media Video (*.wmv)|*.wmv";
            OFD.InitialDirectory = "c:\\";
            OFD.CheckFileExists = true;
            OFD.FileOk += new System.ComponentModel.CancelEventHandler(OFD_FileOk);                        
            OFD.ShowDialog();                       
        }

        void OFD_FileOk(object sender, System.ComponentModel.CancelEventArgs e)
        {
            string FilePath = (sender as OpenFileDialog).FileName;

            this.mediaMainScreen.Source = new Uri(FilePath);

            this.TryPlayMedia();
        }

        private void mediaMainScreen_MediaEnded(object sender, RoutedEventArgs e)
        {
            this.mediaMainScreen.Stop();
            this.MediaPlayerController.SetPlayerControlStates(MediaPlayerController.ENABLED_STATE.Disabled);
            this.MediaPlayerController.SetPlayButtonState(MediaPlayerController.ENABLED_STATE.Enabled); 
        }

        private void mediaMainScreen_MediaFailed(object sender, ExceptionRoutedEventArgs e)
        {
            string Message = "A Media error has occured with the following message:" +
                Environment.NewLine + Environment.NewLine + e.ErrorException.Message;

            MessageBox.Show(Message, "Media Failed", MessageBoxButton.OK, MessageBoxImage.Error);

        }
                        
        private void mediaMainScreen_MediaOpened(object sender, RoutedEventArgs e)
        {
            this.TryPlayMedia();
        }

        private void TryPlayMedia()
        {
            try
            {
                this.mediaMainScreen.Play();

                this.MediaPlayerController.SetPlayerControlStates(MediaPlayerController.ENABLED_STATE.Enabled);
                this.MediaPlayerController.SetPlayButtonState(MediaPlayerController.ENABLED_STATE.Disabled);
                this.MediaPlayerController.SetStopButtonState(MediaPlayerController.ENABLED_STATE.Enabled);

                if (this.mediaMainScreen.CanPause)
                    this.MediaPlayerController.SetPauseButtonState(MediaPlayerController.ENABLED_STATE.Enabled);
                else
                    this.MediaPlayerController.SetPauseButtonState(MediaPlayerController.ENABLED_STATE.Disabled);

            }
            catch (Exception x)
            {

                string Message = " The Media Playback process has failed with the following message:" +
                    Environment.NewLine + Environment.NewLine + x.Message;

                MessageBox.Show(Message, "Media Playback Failed", MessageBoxButton.OK, MessageBoxImage.Error);

            }
        }
        
        void CompositionTarget_Rendering(object sender, EventArgs e)
        {
            if (this.mediaMainScreen.NaturalDuration.HasTimeSpan)
            {
                double Total = this.mediaMainScreen.NaturalDuration.TimeSpan.TotalMilliseconds;
                double Current = this.mediaMainScreen.Position.TotalMilliseconds;

                this.MediaPlayerController.SetPlayerPostion(Current, Total);
            }
        }

	}
}